@@ -27,11 +27,10 @@ import rasterio.crs as rio_crs
 from click.core import ParameterSource
 from geedim import schema, Initialize, version
 from geedim.collection import MaskedCollection
-from geedim.download import BaseImage
+from geedim.download import BaseImage, supported_dtypes
 from geedim.enums import CloudMaskMethod, CompositeMethod, ResamplingMethod
 from geedim.mask import MaskedImage
 from geedim.utils import get_bounds, Spinner
-from rasterio.dtypes import dtype_ranges
 from rasterio.errors import CRSError
 
 logger = logging.getLogger(__name__)
@@ -213,7 +212,7 @@ scale_option = click.option(
     help='Pixel scale (size) to resample image(s) to (m).'
 )
 dtype_option = click.option(
-    '-dt', '--dtype', type=click.Choice(list(dtype_ranges.keys()), case_sensitive=False), default=None,
+    '-dt', '--dtype', type=click.Choice(supported_dtypes, case_sensitive=False), default=None,
     show_default='smallest data type able to represent the range of pixel values.',
     help='Data type to convert image(s) to.'
 )
@@ -487,9 +486,17 @@ cli.add_command(search)
 @mask_option
 @resampling_option
 @scale_offset_option
+@click.option(
+    '-mts', '--max-tile-size', type=click.FLOAT, default=BaseImage._ee_max_tile_size, show_default=True,
+    help='Maximum download tile size (MB).'
+)
+@click.option(
+    '-mtd', '--max-tile-dim', type=click.INT, default=BaseImage._ee_max_tile_dim, show_default=True,
+    help='Maximum download tile dimension (pixels).'
+)
 @click.option('-o', '--overwrite', is_flag=True, default=False, help='Overwrite the destination file if it exists.')
 @click.pass_obj
-def download(obj, image_id, bbox, region, download_dir, mask, overwrite, **kwargs):
+def download(obj, image_id, bbox, region, download_dir, mask, max_tile_size, max_tile_dim, overwrite, **kwargs):
     # @formatter:off
     """
     Download image(s).
@@ -514,7 +521,7 @@ def download(obj, image_id, bbox, region, download_dir, mask, overwrite, **kwarg
         SHADOW_MASK     Mask of cloud shadows.
         CLOUD_MASK      Mask of clouds.
         CLOUDLESS_MASK  Mask of filled & cloud/shadow-free pixels.
-        CLOUD_DIST      Distance to nearest cloud (m).
+        CLOUD_DIST      Distance to nearest cloud (10m).
         ==============  =========================================
 
     Images from other collections, will contain the FILL_MASK band only.
@@ -542,7 +549,10 @@ def download(obj, image_id, bbox, region, download_dir, mask, overwrite, **kwarg
     image_list = _prepare_image_list(obj, mask=mask)
     for im in image_list:
         filename = pathlib.Path(download_dir).joinpath(im.name + '.tif')
-        im.download(filename, overwrite=overwrite, region=obj.region, **kwargs)
+        im.download(
+            filename, region=obj.region, max_tile_size=max_tile_size, max_tile_dim=max_tile_dim, overwrite=overwrite,
+            **kwargs
+        )
 
 
 cli.add_command(download)
@@ -589,7 +599,7 @@ def export(obj, image_id, bbox, region, drive_folder, mask, wait, **kwargs):
         SHADOW_MASK     Mask of cloud shadows.
         CLOUD_MASK      Mask of clouds.
         CLOUDLESS_MASK  Mask of filled & cloud/shadow-free pixels.
-        CLOUD_DIST      Distance to nearest cloud.
+        CLOUD_DIST      Distance to nearest cloud (10m).
         ==============  =========================================
 
     Images from other collections, will contain the FILL_MASK band only.
