@@ -24,15 +24,11 @@ import warnings
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from datetime import datetime
 from itertools import product
-from typing import Tuple, Dict, List, Union, Iterator
+from typing import Tuple, Dict, List, Union, Iterator, Optional
 
 import ee
 import numpy as np
 import rasterio as rio
-from geedim import utils
-from geedim.enums import ResamplingMethod
-from geedim.stac import StacCatalog, StacItem
-from geedim.tile import Tile
 from rasterio.crs import CRS
 from rasterio.enums import Resampling as RioResampling
 from rasterio.windows import Window
@@ -40,13 +36,28 @@ from tqdm import TqdmWarning
 from tqdm.auto import tqdm
 from tqdm.contrib.logging import logging_redirect_tqdm
 
+from geedim import utils
+from geedim.enums import ResamplingMethod
+from geedim.stac import StacCatalog, StacItem
+from geedim.tile import Tile
+
 logger = logging.getLogger(__name__)
 
+supported_dtypes = ['uint8', 'uint16', 'uint32', 'int8', 'int16', 'int32', 'float32', 'float64']
+""" Supported image data types for downloading/exporting. """
+# Note:
+# - while gdal >= 3.5 supports *int64 data type, there is a rasterio bug retrieving the *int64 nodata value,
+# so geedim will not support these types for now.
+# - the ordering of the list above is relevant to the auto dtype and should be: unsigned ints smallest - largest,
+# signed ints smallest to largest, float types smallest to largest.
+
 
 class BaseImage:
     _float_nodata = float('nan')
     _desc_width = 50
     _default_resampling = ResamplingMethod.near
+    _ee_max_tile_size = 32
+    _ee_max_tile_dim = 10000
 
     def __init__(self, ee_image: ee.Image):
         """
@@ -102,7 +113,7 @@ class BaseImage:
         return self.__min_projection
 
     @property
-    def _stac(self) -> Union[StacItem, None]:
+    def _stac(self) -> Optional[StacItem]:
         """ Image STAC info.  None if there is no Earth Engine STAC entry for the image / image's collection. """
         return StacCatalog().get_item(self.id)
 
@@ -119,7 +130,7 @@ class BaseImage:
         self._ee_image = value
 
     @property
-    def id(self) -> Union[str, None]:
+    def id(self) -> Optional[str]:
         """ Earth Engine image ID.  None if the image ``system:id`` property is not set. """
         if self._id:  # avoid a call to getInfo() if _id is set
             return self._id
@@ -127,7 +138,7 @@ class BaseImage:
             return self._ee_info['id'] if 'id' in self._ee_info else None
 
     @property
-    def date(self) -> Union[datetime, None]:
+    def date(self) -> Optional[datetime]:
         """ Image capture date & time.  None if the image ``system:time_start`` property is not set. """
         if 'system:time_start' in self.properties:
             return datetime.utcfromtimestamp(self.properties['system:time_start'] / 1000)
@@ -135,31 +146,31 @@ class BaseImage:
             return None
 
     @property
-    def name(self) -> Union[str, None]:
+    def name(self) -> Optional[str]:
         """ Image name (the :attr:`id` with slashes replaced by dashes). """
         return self.id.replace('/', '-') if self.id else None
 
     @property
-    def crs(self) -> Union[str, None]:
+    def crs(self) -> Optional[str]:
         """
         Image CRS corresponding to minimum scale band, as an EPSG string. None if the image has no fixed projection.
         """
         return self._min_projection['crs']
 
     @property
-    def scale(self) -> Union[float, None]:
+    def scale(self) -> Optional[float]:
         """ Minimum scale (m) of image bands. None if the image has no fixed projection. """
         return self._min_projection['scale']
 
     @property
-    def footprint(self) -> Union[Dict, None]:
+    def footprint(self) -> Optional[Dict]:
         """ Geojson polygon of the image extent.  None if the image is a composite. """
         if ('properties' not in self._ee_info) or ('system:footprint' not in self._ee_info['properties']):
             return None
         return self._ee_info['properties']['system:footprint']
 
     @property
-    def shape(self) -> Union[Tuple[int, int], None]:
+    def shape(self) -> Optional[Tuple[int, int]]:
         """ (row, column) pixel dimensions of the minimum scale band. None if the image has no fixed projection. """
         return self._min_projection['shape']
 
@@ -169,7 +180,7 @@ class BaseImage:
         return len(self._ee_info['bands']) if 'bands' in self._ee_info else None
 
     @property
-    def transform(self) -> Union[rio.Affine, None]:
+    def transform(self) -> Optional[rio.Affine]:
         """ Geo-transform of the minimum scale band. None if the image has no fixed projection. """
         return self._min_projection['transform']
 
@@ -181,7 +192,7 @@ class BaseImage:
         return self._min_dtype
 
     @property
-    def size(self) -> int:
+    def size(self) -> Optional[int]:
         """ Image size (bytes).  None if the image has no fixed projection. """
         if not self.shape:
             return None
@@ -204,7 +215,7 @@ class BaseImage:
         return self._get_band_properties()
 
     @property
-    def refl_bands(self) -> Union[List[str], None]:
+    def refl_bands(self) -> Optional[List[str]]:
         """ List of spectral / reflectance bands, if any. """
         if not self._stac:
             return None
@@ -236,25 +247,38 @@ class BaseImage:
 
     @staticmethod
     def _get_min_dtype(ee_info: Dict) -> str:
-        """ Return the minimum size data type corresponding to a given Earth Engine image info dictionary. """
+        """ Return the minimum size rasterio data type corresponding to a given Earth Engine image info dictionary. """
+        def get_min_int_dtype(band_info: List[Dict]) -> Optional[str]:
+            """ Return the minimum integer dtype for the given band information. """
+            # create a list of integer min/max values
+            int_minmax = [
+                minmax for band_dict in band_info if band_dict['data_type']['precision'] == 'int'
+                for minmax in (int(band_dict['data_type']['min']), int(band_dict['data_type']['max']))
+            ]  # yapf: disable
+
+            if len(int_minmax) == 0:
+                return None
+            min_value = np.nanmin(int_minmax)
+            max_value = np.nanmax(int_minmax)
+
+            for dtype in supported_dtypes[:-2]:
+                if (min_value >= np.iinfo(dtype).min) and (max_value <= np.iinfo(dtype).max):
+                    return dtype
+            return 'float64'
+
+        # TODO: from gdal = 3.5, there is support *int64 in geotiffs, rasterio 1.3.0 had some nodata issues with this,
+        #  int64 support should be added when these issues are resolved
         dtype = None
         if 'bands' in ee_info:
             precisions = np.array([bd['data_type']['precision'] for bd in ee_info['bands']])
-            if all(precisions == 'int'):
-                dtype_minmax = np.array(
-                    [(bd['data_type']['min'], bd['data_type']['max']) for bd in ee_info['bands']], dtype=np.int64
-                )
-                dtype_min = min(0, int(dtype_minmax[:, 0].min()))  # minimum image pixel value
-                dtype_max = max(0, int(dtype_minmax[:, 1].max()))  # maximum image pixel value
-
-                # determine the number of integer bits required to represent the value range
-                bits = 2**np.ceil(np.log2(np.log2(dtype_max - dtype_min)))
-                bits = min(max(bits, 8), 32)  # clamp bits to allowed values
-                dtype = f'{"u" if dtype_min >= 0 else ""}int{int(bits)}'
-            elif any(precisions == 'double'):
+            if any(precisions == 'double'):
                 dtype = 'float64'
+            elif any(precisions == 'float'):
+                # if there are >= 32 integer bits, use float64 to accommodate them, otherwise float32
+                int_dtype = get_min_int_dtype(ee_info['bands'])
+                dtype = 'float32' if (not int_dtype or np.dtype(int_dtype).itemsize < 4) else 'float64'
             else:
-                dtype = 'float32'
+                dtype = get_min_int_dtype(ee_info['bands'])
         return dtype
 
     @staticmethod
@@ -273,7 +297,7 @@ class BaseImage:
         """ Convert the data type of an Earth Engine image to a specified type. """
         conv_dict = dict(
             float32=ee.Image.toFloat, float64=ee.Image.toDouble, uint8=ee.Image.toUint8, int8=ee.Image.toInt8,
-            uint16=ee.Image.toUint16, int16=ee.Image.toInt16, uint32=ee.Image.toUint32, int32=ee.Image.toInt32
+            uint16=ee.Image.toUint16, int16=ee.Image.toInt16, uint32=ee.Image.toUint32, int32=ee.Image.toInt32,
         )
         if dtype not in conv_dict:
             raise TypeError(f'Unsupported dtype: {dtype}')
@@ -360,8 +384,8 @@ class BaseImage:
         resampling : ResamplingMethod, optional
             Resampling method - see :class:`~geedim.enums.ResamplingMethod` for available options.
         dtype: str, optional
-           Convert to this data type (`uint8`, `int8`, `uint16`, `int16`, `uint32`, `int32`, `float32`
-           or `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
+           Convert to this data type (`uint8`, `int8`, `uint16`, `int16`, `uint32`, `int32`, `float32` or
+           `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
         scale_offset: bool, optional
             Whether to apply any EE band scales and offsets to the image.
 
@@ -421,7 +445,7 @@ class BaseImage:
         ee_image, _ = ee_image.prepare_for_export(export_args)
         return BaseImage(ee_image)
 
-    def _prepare_for_download(self, set_nodata: bool = True, **kwargs) -> ('BaseImage', Dict):
+    def _prepare_for_download(self, set_nodata: bool = True, **kwargs) -> Tuple['BaseImage', Dict]:
         """
         Prepare the encapsulated image for tiled GeoTIFF download. Will reproject, resample, clip and convert the image
         according to the provided parameters.
@@ -439,7 +463,7 @@ class BaseImage:
             uint16=0,
             int16=np.iinfo('int16').min,
             uint32=0,
-            int32=np.iinfo('int32').min
+            int32=np.iinfo('int32').min,
         )  # yapf: disable
         nodata = nodata_dict[exp_image.dtype] if set_nodata else None
         profile = dict(
@@ -454,14 +478,27 @@ class BaseImage:
 
     @staticmethod
     def _get_tile_shape(
-        exp_image: 'BaseImage', max_download_size: int = 32 << 20, max_grid_dimension: int = 10000
-    ) -> (Tuple[int, int], int):  # yapf: disable
+        exp_image: 'BaseImage', max_tile_size: Optional[float] = None, max_tile_dim: Optional[int] = None
+    ) -> Tuple[Tuple[int, int], int]:  # yapf: disable
         """
         Return a tile shape and number of tiles for a given BaseImage, such that the tile shape satisfies GEE
         download limits, and is 'square-ish'.
         """
+        # convert max_tile_size from MB to bytes & set to EE default if None
+        if max_tile_size and (max_tile_size > BaseImage._ee_max_tile_size):
+            raise ValueError(
+                f'`max_tile_size` must be less than or equal to the Earth Engine download size limit of '
+                f'{BaseImage._ee_max_tile_size} MB.'
+            )
+        max_tile_size = int((max_tile_size * (1 << 20)) if max_tile_size else (BaseImage._ee_max_tile_size << 20))
+        if max_tile_dim and (max_tile_dim > BaseImage._ee_max_tile_dim):
+            raise ValueError(
+                f'`max_tile_dim` must be less than or equal to the Earth Engine download limit of '
+                f'{BaseImage._ee_max_tile_size} pixels.'
+            )
+        max_tile_dim = max_tile_dim or BaseImage._ee_max_tile_dim   # set max_tile_dim to EE default if None
 
-        # find the total number of tiles the image must be divided into to satisfy max_download_size
+        # find the total number of tiles the image must be divided into to satisfy max_tile_size
         image_shape = np.array(exp_image.shape, dtype='int64')
         dtype_size = np.dtype(exp_image.dtype).itemsize
         image_size = exp_image.size
@@ -475,13 +512,13 @@ class BaseImage:
         num_tile_shape = np.array([1, 1], dtype='int64')
         tile_size = image_size
         tile_shape = image_shape
-        while tile_size >= max_download_size:
+        while tile_size >= max_tile_size:
             div_axis = np.argmax(tile_shape)
             num_tile_shape[div_axis] += 1  # increase the num tiles down the longest dimension of tile_shape
             tile_shape = np.ceil(image_shape / num_tile_shape).astype('int64')
             tile_size = tile_shape[0] * tile_shape[1] * pixel_size
 
-        tile_shape[tile_shape > max_grid_dimension] = max_grid_dimension
+        tile_shape[tile_shape > max_tile_dim] = max_tile_dim
         num_tiles = int(np.product(np.ceil(image_shape / tile_shape)))
         tile_shape = tuple(tile_shape.tolist())
         return tile_shape, num_tiles
@@ -531,7 +568,7 @@ class BaseImage:
             dataset.update_tags(band_i + 1, **clean_band_dict)
 
     @staticmethod
-    def _tiles(exp_image: 'BaseImage', tile_shape: Tuple[int, int] = None) -> Iterator[Tile]:
+    def _tiles(exp_image: 'BaseImage', tile_shape: Tuple[int, int]) -> Iterator[Tile]:
         """
         Iterator over downloadable image tiles.
 
@@ -541,17 +578,15 @@ class BaseImage:
         ----------
         exp_image: BaseImage
             Image to tile.
-        tile_shape: Tuple[int, int], optional
-            (row, column) tile shape to use (pixels). Defaults to calculate an auto tile shape that satisfies the
-            Earth Engine download size limit.
+        tile_shape: Tuple[int, int]
+            (row, column) tile shape to use (pixels). Use :meth:`BaseImage._get_tile_shape` to find a tile shape that
+            satisfies the Earth Engine download limit for :param:`exp_image`.
 
         Yields
         -------
         Tile
             An image tile that can be downloaded.
         """
-        if not tile_shape:
-            tile_shape, num_tiles = BaseImage._get_tile_shape(exp_image)
 
         # split the image up into tiles of at most `tile_shape` dimension
         image_shape = exp_image.shape
@@ -623,8 +658,8 @@ class BaseImage:
         resampling : ResamplingMethod, optional
             Resampling method - see :class:`~geedim.enums.ResamplingMethod` for available options.
         dtype: str, optional
-           Convert to this data type (`uint8`, `int8`, `uint16`, `int16`, `uint32`, `int32`, `float32`
-           or `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
+           Convert to this data type (`uint8`, `int8`, `uint16`, `int16`, `uint32`, `int32`, `float32` or
+           `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
         scale_offset: bool, optional
             Whether to apply any EE band scales and offsets to the image.
 
@@ -648,7 +683,10 @@ class BaseImage:
             self.monitor_export(task)
         return task
 
-    def download(self, filename: Union[pathlib.Path, str], overwrite: bool = False, num_threads: int = None, **kwargs):
+    def download(
+        self, filename: Union[pathlib.Path, str], overwrite: bool = False, num_threads: Optional[int] = None,
+        max_tile_size: Optional[float] = None, max_tile_dim: Optional[int] = None, **kwargs
+    ):
         """
         Download the encapsulated image to a GeoTiff file.
 
@@ -665,6 +703,10 @@ class BaseImage:
             Overwrite the destination file if it exists.
         num_threads: int, optional
             Number of tiles to download concurrently.  Defaults to a sensible auto value.
+        max_tile_size: int, optional
+            Maximum tile size (MB).  If None, defaults to the Earth Engine download size limit (32 MB).
+        max_tile_dim: int, optional
+            Maximum tile width/height (pixels).  If None, defaults to Earth Engine download limit (10000).
         region : dict, ee.Geometry, optional
             Region defined by geojson polygon in WGS84.  Defaults to the entire image granule.
         crs : str, optional
@@ -674,10 +716,10 @@ class BaseImage:
             Resample image(s) to this pixel scale (size) (m).  Where image bands have different scales,
             all are resampled to this scale.  Defaults to the minimum scale of image bands.
         resampling : ResamplingMethod, optional
-            Resampling method - see :class:`~geedim.enums.ResamplingMethod` for available options.
+            Resampling method - see :class:`~geedim.enums.ResamplingMethod` for available options.  Defaults to `near`.
         dtype: str, optional
-            Convert to this data type (`uint8`, `int8`, `uint16`, `int16`, `uint32`, `int32`, `float32`
-            or `float64`).  Defaults to auto select a minimum size type that can represent the range of pixel values.
+           Convert to this data type (`uint8`, `int8`, `uint16`, `int16`, `uint32`, `int32`, `float32` or
+           `float64`). Defaults to auto select a minimal type that can represent the range of pixel values.
         scale_offset: bool, optional
             Whether to apply any EE band scales and offsets to the image.
         """
@@ -695,7 +737,7 @@ class BaseImage:
         exp_image, profile = self._prepare_for_download(**kwargs)
 
         # get the dimensions of an image tile that will satisfy GEE download limits
-        tile_shape, num_tiles = self._get_tile_shape(exp_image)
+        tile_shape, num_tiles = self._get_tile_shape(exp_image, max_tile_size=max_tile_size, max_tile_dim=max_tile_dim)
 
         # find raw size of the download data (less than the actual download size as the image data is zipped in a
         # compressed geotiff)
@@ -746,9 +788,10 @@ class BaseImage:
                         future.result()
                 except Exception as ex:
                     logger.info(f'Exception: {str(ex)}\nCancelling...')
-                    executor.shutdown(wait=False, cancel_futures=True)
+                    executor.shutdown(wait=False)
                     raise ex
 
+            bar.update(bar.total - bar.n)   # ensure the bar reaches 100%
             # populate GeoTIFF metadata and build overviews
             self._write_metadata(out_ds)
             self._build_overviews(out_ds)
